import cv2
import numpy as np


# Non Maxima Suppression
def nms(S):
    W1 = np.array([\
        (1, 0, 0),\
        (0, -1, 0),\
        (0, 0, 0)]\
        ,dtype = float)
    C1 = cv2.filter2D(S, cv2.CV_32F, W1)<0 
    W2 = np.array([\
        (0, 1, 0),\
        (0, -1, 0),\
        (0, 0, 0)]\
        ,dtype = float)
    C2 = cv2.filter2D(S, cv2.CV_32F, W2)<0 
    W3 = np.array([\
        (0, 0, 1),\
        (0, -1, 0),\
        (0, 0, 0)]\
        ,dtype = float)
    C3 = cv2.filter2D(S, cv2.CV_32F, W3)<0 
    W4 = np.array([\
        (0, 0, 0),\
        (1, -1, 0),\
        (0, 0, 0)]\
        ,dtype = float)
    C4 = cv2.filter2D(S, cv2.CV_32F, W4)<0 
    W5 = np.array([\
        (0, 0, 0),\
        (0, -1, 1),\
        (0, 0, 0)]\
        ,dtype = float)
    C5 = cv2.filter2D(S, cv2.CV_32F, W5)<0 
    W6 = np.array([\
        (0, 0, 0),\
        (0, -1, 0),\
        (1, 0, 0)]\
        ,dtype = float)
    C6 = cv2.filter2D(S, cv2.CV_32F, W6)<0 
    W7 = np.array([\
        (0, 0, 0),\
        (0, -1, 0),\
        (0, 1, 0)]\
        ,dtype = float)
    C7 = cv2.filter2D(S, cv2.CV_32F, W7)<0 
    W8 = np.array([\
        (0, 0, 0),\
        (0, -1, 0),\
        (0, 0, 1)]\
        ,dtype = float)
    C8 = cv2.filter2D(S, cv2.CV_32F, W8)<0 
    Sf = C1 & C2 & C3 & C4 & C5 & C6 & C7 & C8
    return Sf

def harris(img,k=0.04):
  #Im should be a 1 channel image
  # Define some Kernels 
  Wgx = np.array([\
      (-1, 0, 1),\
      (-2, 0, 2),\
      (-1, 0, 1)]\
      ,dtype = float)
  Wf = np.ones((5,5))/25
  iGx = cv2.filter2D(img, cv2.CV_32F, Wgx)
  # Apply the windows filter
  iGy = cv2.filter2D(img, cv2.CV_32F, np.transpose(Wgx))
  iGxx = np.multiply(iGx,iGx)
  iGyy = np.multiply(iGy,iGy)
  iGxy = np.multiply(iGx,iGy)
  WiGxx = cv2.filter2D(iGxx, cv2.CV_32F, Wf)
  WiGyy = cv2.filter2D(iGyy, cv2.CV_32F, Wf)
  WiGxy = cv2.filter2D(iGxy, cv2.CV_32F, Wf)
  # Compute trace and det
  tr = WiGxx + WiGyy
  D = np.multiply(WiGxx,WiGyy) - np.multiply(WiGxy,WiGxy)
  # Compute Harris score
  S = D - k*(np.multiply(tr,tr))
  # threshold according to a fraction of max scores
  fr = 0.01
  S2 = S*(S>fr*np.max(S.flatten()))
  # or threshold according to a fix value
  #non maxima suppression
  S3 = nms(S2)
  # get coordinates
  ptsl , ptsc  = np.nonzero(S3)
  return np.transpose(np.concatenate(([ptsl],[ptsc]),axis=0))
